@@ -12,6 +12,7 @@
 #include <algorithm>
 #include <functional>
 
+#include <OpenThreads/Atomic>
 #include <OpenThreads/Mutex>
 #include <OpenThreads/ReentrantMutex>
 #include <OpenThreads/ScopedLock>
@@ -42,6 +43,8 @@
 #include <simgear/props/props.hxx>
 #include <simgear/structure/SGBinding.hxx>
 #include <simgear/scene/material/EffectGeode.hxx>
+#include <simgear/scene/material/EffectCullVisitor.hxx>
+#include <simgear/scene/util/DeletionManager.hxx>
 #include <simgear/scene/util/OsgMath.hxx>
 #include <simgear/scene/util/SGNodeMasks.hxx>
 #include <simgear/scene/util/SGSceneUserData.hxx>
@@ -69,51 +72,6 @@ using namespace simgear;
 // Static utility functions.
 ////////////////////////////////////////////////////////////////////////
 
-/**
- * Set up the transform matrix for a spin or rotation.
- */
-static void
-set_rotation (osg::Matrix &matrix, double position_deg,
-              const SGVec3d &center, const SGVec3d &axis)
-{
-  double temp_angle = -SGMiscd::deg2rad(position_deg);
-  
-  double s = sin(temp_angle);
-  double c = cos(temp_angle);
-  double t = 1 - c;
-  
-  // axis was normalized at load time 
-  // hint to the compiler to put these into FP registers
-  double x = axis[0];
-  double y = axis[1];
-  double z = axis[2];
-  
-  matrix(0, 0) = t * x * x + c ;
-  matrix(0, 1) = t * y * x - s * z ;
-  matrix(0, 2) = t * z * x + s * y ;
-  matrix(0, 3) = 0;
-  
-  matrix(1, 0) = t * x * y + s * z ;
-  matrix(1, 1) = t * y * y + c ;
-  matrix(1, 2) = t * z * y - s * x ;
-  matrix(1, 3) = 0;
-  
-  matrix(2, 0) = t * x * z - s * y ;
-  matrix(2, 1) = t * y * z + s * x ;
-  matrix(2, 2) = t * z * z + c ;
-  matrix(2, 3) = 0;
-  
-  // hint to the compiler to put these into FP registers
-  x = center[0];
-  y = center[1];
-  z = center[2];
-  
-  matrix(3, 0) = x - x*matrix(0, 0) - y*matrix(1, 0) - z*matrix(2, 0);
-  matrix(3, 1) = y - x*matrix(0, 1) - y*matrix(1, 1) - z*matrix(2, 1);
-  matrix(3, 2) = z - x*matrix(0, 2) - y*matrix(1, 2) - z*matrix(2, 2);
-  matrix(3, 3) = 1;
-}
-
 /**
  * Set up the transform matrix for a translation.
  */
@@ -466,7 +424,7 @@ SGAnimation::animate(osg::Node* node, const SGPropertyNode* configNode,
     SGTranslateAnimation animInst(configNode, modelRoot);
     animInst.apply(node);
   } else if (type == "light") {
-    SGLightAnimation animInst(configNode, modelRoot, path, i);
+    SGLightAnimation animInst(configNode, modelRoot, options, path, i);
     animInst.apply(node);
   } else if (type == "null" || type == "none" || type.empty()) {
     SGGroupAnimation animInst(configNode, modelRoot);
@@ -655,7 +613,9 @@ public:
                  SGExpressiond const* animationValue) :
     _condition(condition),
     _animationValue(animationValue)
-  { }
+  {
+      setName("SGTranslateAnimation::UpdateCallback");
+  }
   virtual void operator()(osg::Node* node, osg::NodeVisitor* nv)
   {
     if (!_condition || _condition->test()) {
@@ -722,60 +682,172 @@ SGTranslateAnimation::createAnimationGroup(osg::Group& parent)
 // Implementation of rotate/spin animation
 ////////////////////////////////////////////////////////////////////////
 
-class SGRotateAnimation::UpdateCallback : public osg::NodeCallback {
+class SGRotAnimTransform : public SGRotateTransform
+{
 public:
-  UpdateCallback(SGCondition const* condition,
-                 SGExpressiond const* animationValue) :
-    _condition(condition),
-    _animationValue(animationValue)
-  { }
-  virtual void operator()(osg::Node* node, osg::NodeVisitor* nv)
-  {
-    if (!_condition || _condition->test()) {
-      SGRotateTransform* transform;
-      transform = static_cast<SGRotateTransform*>(node);
-      transform->setAngleDeg(_animationValue->getValue());
-    }
-    traverse(node, nv);
-  }
-public:
-  SGSharedPtr<SGCondition const> _condition;
-  SGSharedPtr<SGExpressiond const> _animationValue;
+    SGRotAnimTransform();
+    SGRotAnimTransform(const SGRotAnimTransform&,
+                       const osg::CopyOp& copyop = osg::CopyOp::SHALLOW_COPY);
+    META_Node(simgear, SGRotAnimTransform);
+    virtual bool computeLocalToWorldMatrix(osg::Matrix& matrix,
+                                           osg::NodeVisitor* nv) const;
+    virtual bool computeWorldToLocalMatrix(osg::Matrix& matrix,
+                                           osg::NodeVisitor* nv) const;
+    SGSharedPtr<SGCondition const> _condition;
+    SGSharedPtr<SGExpressiond const> _animationValue;
+    // used when condition is false
+    mutable double _lastAngle;
 };
 
-class SGRotateAnimation::SpinUpdateCallback : public osg::NodeCallback {
+SGRotAnimTransform::SGRotAnimTransform()
+    : _lastAngle(0.0)
+{
+}
+
+SGRotAnimTransform::SGRotAnimTransform(const SGRotAnimTransform& rhs,
+                                       const osg::CopyOp& copyop)
+    : SGRotateTransform(rhs, copyop), _condition(rhs._condition),
+      _animationValue(rhs._animationValue), _lastAngle(rhs._lastAngle)
+{
+}
+
+bool SGRotAnimTransform::computeLocalToWorldMatrix(osg::Matrix& matrix,
+                                                   osg::NodeVisitor* nv) const
+{
+    double angle = 0.0;
+    if (!_condition || _condition->test()) {
+        angle = _animationValue->getValue();
+        _lastAngle = angle;
+    } else {
+        angle = _lastAngle;
+    }
+    double angleRad = SGMiscd::deg2rad(angle);
+    if (_referenceFrame == RELATIVE_RF) {
+        // FIXME optimize
+        osg::Matrix tmp;
+        set_rotation(tmp, angleRad, getCenter(), getAxis());
+        matrix.preMult(tmp);
+    } else {
+        osg::Matrix tmp;
+        SGRotateTransform::set_rotation(tmp, angleRad, getCenter(), getAxis());
+        matrix = tmp;
+    }
+    return true;
+}
+
+bool SGRotAnimTransform::computeWorldToLocalMatrix(osg::Matrix& matrix,
+                                                   osg::NodeVisitor* nv) const
+{
+    double angle = 0.0;
+    if (!_condition || _condition->test()) {
+        angle = _animationValue->getValue();
+        _lastAngle = angle;
+    } else {
+        angle = _lastAngle;
+    }
+    double angleRad = SGMiscd::deg2rad(angle);
+    if (_referenceFrame == RELATIVE_RF) {
+        // FIXME optimize
+        osg::Matrix tmp;
+        set_rotation(tmp, -angleRad, getCenter(), getAxis());
+        matrix.postMult(tmp);
+    } else {
+        osg::Matrix tmp;
+        set_rotation(tmp, -angleRad, getCenter(), getAxis());
+        matrix = tmp;
+    }
+    return true;
+}
+
+// Cull callback
+class SpinAnimCallback : public osg::NodeCallback {
 public:
-  SpinUpdateCallback(SGCondition const* condition,
-                     SGExpressiond const* animationValue) :
+    SpinAnimCallback(SGCondition const* condition,
+                       SGExpressiond const* animationValue,
+          double initialValue = 0.0) :
     _condition(condition),
     _animationValue(animationValue),
-    _lastTime(-1)
-  { }
-  virtual void operator()(osg::Node* node, osg::NodeVisitor* nv)
-  {
-    if (!_condition || _condition->test()) {
-      SGRotateTransform* transform;
-      transform = static_cast<SGRotateTransform*>(node);
-
-      double t = nv->getFrameStamp()->getReferenceTime();
-      double dt = 0;
-      if (0 <= _lastTime)
-        dt = t - _lastTime;
-      _lastTime = t;
-      double velocity_rpms = _animationValue->getValue()/60;
-      double angle = transform->getAngleDeg();
-      angle += dt*velocity_rpms*360;
-      angle -= 360*floor(angle/360);
-      transform->setAngleDeg(angle);
-    }
-    traverse(node, nv);
-  }
+    _initialValue(initialValue)
+    {}
+    virtual void operator()(osg::Node* node, osg::NodeVisitor* nv);
 public:
-  SGSharedPtr<SGCondition const> _condition;
-  SGSharedPtr<SGExpressiond const> _animationValue;
-  double _lastTime;
+    SGSharedPtr<SGCondition const> _condition;
+    SGSharedPtr<SGExpressiond const> _animationValue;
+    double _initialValue;
+protected:
+    // This cull callback can run in different threads if there is
+    // more than one camera. It is probably safe to overwrite the
+    // reference values in multiple threads, but we'll provide a
+    // threadsafe way to manage those values just to be safe.
+    struct ReferenceValues : public osg::Referenced
+    {
+        ReferenceValues(double t, double rot, double vel)
+            : _time(t), _rotation(rot), _rotVelocity(vel)
+        {
+        }
+        double _time;
+        double _rotation;
+        double _rotVelocity;
+    };
+    OpenThreads::AtomicPtr _referenceValues;
 };
 
+void SpinAnimCallback::operator()(osg::Node* node, osg::NodeVisitor* nv)
+{
+    using namespace osg;
+    SGRotateTransform* transform = static_cast<SGRotateTransform*>(node);
+    EffectCullVisitor* cv = dynamic_cast<EffectCullVisitor*>(nv);
+    if (!cv)
+        return;
+    if (!_condition || _condition->test()) {
+        double t = nv->getFrameStamp()->getReferenceTime();
+        double rps = _animationValue->getValue() / 60.0;
+        ref_ptr<ReferenceValues>
+            refval(static_cast<ReferenceValues*>(_referenceValues.get()));
+    if (!refval || refval->_rotVelocity != rps) {
+            ref_ptr<ReferenceValues> newref;
+            if (!refval.valid()) {
+                // initialization
+                newref = new ReferenceValues(t, 0.0, rps);
+            } else {
+                double newRot = refval->_rotation + (t - refval->_time) * refval->_rotVelocity;
+                newref = new ReferenceValues(t, newRot, rps);
+            }
+            // increment reference pointer, because it will be stored
+            // naked in _referenceValues.
+            newref->ref();
+            if (_referenceValues.assign(newref, refval)) {
+                if (refval.valid()) {
+                    DeletionManager::instance()->addStaleObject(refval.get());
+                    refval->unref();
+                }
+            } else {
+                // Another thread installed new values before us
+                newref->unref();
+            }
+            // Whatever happened, we can use the reference values just
+            // calculated.
+            refval = newref;
+        }
+        double rotation = refval->_rotation + (t - refval->_time) * rps;
+        double intPart;
+        double rot = modf(rotation, &intPart);
+        double angle = rot * 2.0 * osg::PI;
+        const SGVec3d& sgcenter = transform->getCenter();
+        const SGVec3d& sgaxis = transform->getAxis();
+        Matrixd mat = Matrixd::translate(-sgcenter[0], -sgcenter[1], -sgcenter[2])
+            * Matrixd::rotate(angle, sgaxis[0], sgaxis[1], sgaxis[2])
+            * Matrixd::translate(sgcenter[0], sgcenter[1], sgcenter[2])
+            * *cv->getModelViewMatrix();
+        ref_ptr<RefMatrix> refmat = new RefMatrix(mat);
+        cv->pushModelViewMatrix(refmat.get(), transform->getReferenceFrame());
+        traverse(transform, nv);
+        cv->popModelViewMatrix();
+    } else {
+        traverse(transform, nv);
+    }
+}
+
 SGRotateAnimation::SGRotateAnimation(const SGPropertyNode* configNode,
                                      SGPropertyNode* modelRoot) :
   SGAnimation(configNode, modelRoot)
@@ -792,7 +864,6 @@ SGRotateAnimation::SGRotateAnimation(const SGPropertyNode* configNode,
     _initialValue = _animationValue->getValue();
   else
     _initialValue = 0;
-
   _center = SGVec3d::zeros();
   if (configNode->hasValue("axis/x1-m")) {
     SGVec3d v1, v2;
@@ -820,21 +891,28 @@ SGRotateAnimation::SGRotateAnimation(const SGPropertyNode* configNode,
 osg::Group*
 SGRotateAnimation::createAnimationGroup(osg::Group& parent)
 {
-  SGRotateTransform* transform = new SGRotateTransform;
-  transform->setName("rotate animation");
-  if (_isSpin) {
-    SpinUpdateCallback* uc;
-    uc = new SpinUpdateCallback(_condition, _animationValue);
-    transform->setUpdateCallback(uc);
-  } else if (_animationValue || !_animationValue->isConst()) {
-    UpdateCallback* uc = new UpdateCallback(_condition, _animationValue);
-    transform->setUpdateCallback(uc);
-  }
-  transform->setCenter(_center);
-  transform->setAxis(_axis);
-  transform->setAngleDeg(_initialValue);
-  parent.addChild(transform);
-  return transform;
+    if (_isSpin) {
+        SGRotateTransform* transform = new SGRotateTransform;
+        transform->setName("spin rotate animation");
+        SpinAnimCallback* cc;
+        cc = new SpinAnimCallback(_condition, _animationValue, _initialValue);
+        transform->setCullCallback(cc);
+        transform->setCenter(_center);
+        transform->setAxis(_axis);
+        transform->setAngleDeg(_initialValue);
+        parent.addChild(transform);
+        return transform;
+    } else {
+        SGRotAnimTransform* transform = new SGRotAnimTransform;
+        transform->setName("rotate animation");
+        transform->_condition = _condition;
+        transform->_animationValue = _animationValue;
+        transform->_lastAngle = _initialValue;
+        transform->setCenter(_center);
+        transform->setAxis(_axis);
+        parent.addChild(transform);
+        return transform;
+    }
 }
 
 
@@ -851,6 +929,7 @@ public:
     _animationValue[0] = animationValue[0];
     _animationValue[1] = animationValue[1];
     _animationValue[2] = animationValue[2];
+    setName("SGScaleAnimation::UpdateCallback");
   }
   virtual void operator()(osg::Node* node, osg::NodeVisitor* nv)
   {
@@ -1373,7 +1452,9 @@ public:
     _maxAnimationValue(maxAnimationValue),
     _minStaticValue(minValue),
     _maxStaticValue(maxValue)
-  {}
+  {
+      setName("SGRangeAnimation::UpdateCallback");
+  }
   virtual void operator()(osg::Node* node, osg::NodeVisitor* nv)
   {
     osg::LOD* lod = static_cast<osg::LOD*>(node);
@@ -1629,7 +1710,9 @@ public:
   UpdateCallback(const SGPropertyNode* configNode, const SGExpressiond* v) :
     _prev_value(-1),
     _animationValue(v)
-  { }
+  {
+      setName("SGBlendAnimation::UpdateCallback");
+  }
   virtual void operator()(osg::Node* node, osg::NodeVisitor* nv)
   {
     double blend = _animationValue->getValue();
@@ -1708,6 +1791,7 @@ public:
                                        rNode->getDoubleValue( "max", 1));
       }
     }
+    setName("SGTimedAnimation::UpdateCallback");
   }
   virtual void operator()(osg::Node* node, osg::NodeVisitor* nv)
   {
@@ -1796,7 +1880,9 @@ class SGShadowAnimation::UpdateCallback : public osg::NodeCallback {
 public:
   UpdateCallback(const SGCondition* condition) :
     _condition(condition)
-  {}
+  {
+      setName("SGShadowAnimation::UpdateCallback");
+  }
   virtual void operator()(osg::Node* node, osg::NodeVisitor* nv)
   {
     if (_condition->test())
@@ -1876,7 +1962,8 @@ public:
   virtual void transform(osg::Matrix& matrix)
   {
     osg::Matrix tmp;
-    set_rotation(tmp, _value, _center, _axis);
+    SGRotateTransform::set_rotation(tmp, SGMiscd::deg2rad(_value), _center,
+                                    _axis);
     matrix.preMult(tmp);
   }
 private:
@@ -1889,7 +1976,9 @@ class SGTexTransformAnimation::UpdateCallback :
 public:
   UpdateCallback(const SGCondition* condition) :
     _condition(condition)
-  { }
+  {
+      setName("SGTexTransformAnimation::UpdateCallback");
+  }
   virtual void operator () (osg::StateAttribute* sa, osg::NodeVisitor*)
   {
     if (!_condition || _condition->test()) {
