@@ -515,7 +515,7 @@ SGLightFactory::getOdal(const SGLightBin& lights)
   Effect* effect = getLightEffect(10.0f, osg::Vec3(1.0, 0.0001, 0.00000001),
                                   6.0, 10.0, false);
   // centerline lights
-  for (int i = lights.getNumLights() - 1; 2 <= i; --i) {
+  for (int i = lights.getNumLights(); i > 1; --i) {
     EffectGeode* egeode = new EffectGeode;
     egeode->setEffect(effect);
     egeode->addDrawable(getLightDrawable(lights.getLight(i)));
@@ -532,7 +532,7 @@ SGLightFactory::getOdal(const SGLightBin& lights)
   sequence->addChild(group, flashTime);
 
   // add an extra empty group for a break
-  sequence->addChild(new osg::Group, 9 + 1e-1*sg_random());
+  sequence->addChild(new osg::Group, 2 + 1e-1*sg_random());
   sequence->setInterval(osg::Sequence::LOOP, 0, -1);
   sequence->setDuration(1.0f, -1);
   sequence->setMode(osg::Sequence::START);
@@ -548,26 +548,54 @@ SGLightFactory::getHoldShort(const SGDirectionalLightBin& lights)
   if (lights.getNumLights() < 2)
     return 0;
 
-  float flashTime = 2;
+  sg_srandom(unsigned(lights.getLight(0).position[0]));
+  float flashTime = 1 + 0.1 * sg_random();
+  osg::Sequence* sequence = new osg::Sequence;
+
+  // start with lights off
+  sequence->addChild(new osg::Group, flashTime);
+  // ...and increase the lights in steps
+  for (int i = 2; i < 7; i+=2) {
+      Effect* effect = getLightEffect(i, osg::Vec3(1, 0.001, 0.000002),
+                                      0.0f, i, true);
+      EffectGeode* egeode = new EffectGeode;
+      for (unsigned int j = 0; j < lights.getNumLights(); ++j) {
+          egeode->addDrawable(getLightDrawable(lights.getLight(j)));
+          egeode->setEffect(effect);
+      }
+      sequence->addChild(egeode, (i==6) ? flashTime : 0.1);
+  }
+
+  sequence->setInterval(osg::Sequence::SWING, 0, -1);
+  sequence->setDuration(1.0f, -1);
+  sequence->setMode(osg::Sequence::START);
+
+  return sequence;
+}
+
+// Alternating runway guard lights ("wig-wag")
+osg::Node*
+SGLightFactory::getGuard(const SGDirectionalLightBin& lights)
+{
+  if (lights.getNumLights() < 2)
+    return 0;
+
+  // generate a repeatable random seed
+  sg_srandom(unsigned(lights.getLight(0).position[0]));
+  float flashTime = 1.0f + 1*sg_random();
   osg::Sequence* sequence = new osg::Sequence;
   sequence->setDefaultTime(flashTime);
-  Effect* effect = getLightEffect(6, osg::Vec3(1, 0.001, 0.000002),
-                                  0, 6, true);
-  // Lights on
-  EffectGeode* egeode = new EffectGeode;
-  for (int i = lights.getNumLights(); 0 <= i; --i) {
+  Effect* effect = getLightEffect(10.0f, osg::Vec3(1.0, 0.001, 0.000002),
+                                  0.0f, 8.0f, true);
+  for (unsigned int i = 0; i < lights.getNumLights(); ++i) {
+    EffectGeode* egeode = new EffectGeode;
     egeode->setEffect(effect);
     egeode->addDrawable(getLightDrawable(lights.getLight(i)));
+    sequence->addChild(egeode, flashTime);
   }
-  sequence->addChild(egeode, flashTime);
-
-  // Lights off
-  sequence->addChild(new osg::Group, flashTime);
-
   sequence->setInterval(osg::Sequence::LOOP, 0, -1);
   sequence->setDuration(1.0f, -1);
   sequence->setMode(osg::Sequence::START);
   sequence->setSync(true);
-
   return sequence;
 }
