@@ -99,7 +99,7 @@ namespace UpdateInterval
     static const double SuccessfulAttempt = 24*60*60;
     // interval in seconds to allow another update after a failed attempt (10 minutes)
     static const double FailedAttempt     = 10*60;
-};
+}
 
 typedef map<string,time_t> CompletedTiles;
 
@@ -110,7 +110,7 @@ string stripPath(string path)
 {
     // svn doesn't like trailing white-spaces or path separators - strip them!
     path = simgear::strutils::strip(path);
-    int slen = path.length();
+    size_t slen = path.length();
     while ((slen>0)&&
             ((path[slen-1]=='/')||(path[slen-1]=='\\')))
     {
@@ -597,8 +597,15 @@ int SGTerraSync::SvnThread::svnClientSetup(void)
         return EXIT_FAILURE;
 #endif
 
-    apr_pool_t *pool;
-    apr_pool_create(&pool, NULL);
+    apr_pool_t *pool = NULL;
+    
+    apr_allocator_t* allocator = NULL;
+    int aprErr = apr_allocator_create(&allocator);
+    if (aprErr != APR_SUCCESS)
+        return EXIT_FAILURE;
+    
+    apr_pool_create_ex(&pool, NULL /* parent pool */, NULL /* abort func */, allocator);
+    
     svn_error_t *err = NULL;
     SVN_VERSION_DEFINE(_svn_version);
     err = svn_ver_check_list(&_svn_version, mysvn_checklist);
